@@ -41,7 +41,10 @@
 #include <X11/XKBlib.h>
 #include <X11/extensions/XKBstr.h>
 
-#include <libgnome/gnome-help.h>
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
+#include <X11/extensions/XInput.h>
+#include <X11/extensions/XIproto.h>
+#endif
 
 #ifdef HAVE_LIBNOTIFY
 #include <libnotify/notify.h>
@@ -65,6 +68,7 @@ struct GsdA11yKeyboardManagerPrivate
         GtkWidget *slowkeys_alert;
         GtkWidget *preferences_dialog;
         GtkStatusIcon *status_icon;
+        XkbDescRec *original_xkb_desc;
 
         guint      gconf_notify;
 
@@ -73,15 +77,12 @@ struct GsdA11yKeyboardManagerPrivate
 #endif /* HAVE_LIBNOTIFY */
 };
 
-#define GSD_KBD_A11Y_ERROR gsd_kbd_a11y_error_quark ()
-
-enum {
-        GSD_KBD_A11Y_ERROR_NOT_AVAILABLE
-};
-
 static void     gsd_a11y_keyboard_manager_class_init  (GsdA11yKeyboardManagerClass *klass);
 static void     gsd_a11y_keyboard_manager_init        (GsdA11yKeyboardManager      *a11y_keyboard_manager);
 static void     gsd_a11y_keyboard_manager_finalize    (GObject             *object);
+static void     gsd_a11y_keyboard_manager_ensure_status_icon (GsdA11yKeyboardManager *manager);
+static void     set_server_from_gconf (GsdA11yKeyboardManager *manager,
+				       GConfClient            *client);
 
 G_DEFINE_TYPE (GsdA11yKeyboardManager, gsd_a11y_keyboard_manager, G_TYPE_OBJECT)
 
@@ -94,28 +95,84 @@ static gpointer manager_object = NULL;
 #define d(str)          do { } while (0)
 #endif
 
-static GQuark
-gsd_kbd_a11y_error_quark (void)
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
+static GdkFilterReturn
+devicepresence_filter (GdkXEvent *xevent,
+                       GdkEvent  *event,
+                       gpointer   data)
 {
-        return g_quark_from_static_string ("gsd-kbd-a11y-error-quark");
+        XEvent *xev = (XEvent *) xevent;
+        XEventClass class_presence;
+        int xi_presence;
+
+        DevicePresence (gdk_x11_get_default_xdisplay (), xi_presence, class_presence);
+
+        if (xev->type == xi_presence)
+        {
+            XDevicePresenceNotifyEvent *dpn = (XDevicePresenceNotifyEvent *) xev;
+            if (dpn->devchange == DeviceEnabled) {
+                GConfClient *client;
+		client = gconf_client_get_default ();
+		set_server_from_gconf (data, client);
+                g_object_unref (client);
+	    }
+        }
+        return GDK_FILTER_CONTINUE;
 }
 
+static gboolean
+supports_xinput_devices (void)
+{
+        gint op_code, event, error;
+
+        return XQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                "XInputExtension",
+                                &op_code,
+                                &event,
+                                &error);
+}
+
+static void
+set_devicepresence_handler (GsdA11yKeyboardManager *manager)
+{
+        Display *display;
+        XEventClass class_presence;
+        int xi_presence;
+
+        if (!supports_xinput_devices ())
+                return;
+
+        display = gdk_x11_get_default_xdisplay ();
+
+        gdk_error_trap_push ();
+        DevicePresence (display, xi_presence, class_presence);
+        /* FIXME:
+         * Note that this might overwrite other events, see:
+         * https://bugzilla.gnome.org/show_bug.cgi?id=610245#c2
+         **/
+        XSelectExtensionEvent (display,
+                               RootWindow (display, DefaultScreen (display)),
+                               &class_presence, 1);
+
+        gdk_flush ();
+        if (!gdk_error_trap_pop ())
+                gdk_window_add_filter (NULL, devicepresence_filter, manager);
+}
+#endif
+
 static gboolean
 xkb_enabled (GsdA11yKeyboardManager *manager)
 {
         gboolean have_xkb;
         int opcode, errorBase, major, minor;
 
-        gdk_error_trap_push ();
-        have_xkb = XkbQueryExtension (GDK_DISPLAY (),
+        have_xkb = XkbQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                       &opcode,
                                       &manager->priv->xkbEventBase,
                                       &errorBase,
                                       &major,
                                       &minor)
-                && XkbUseExtension (GDK_DISPLAY (), &major, &minor);
-        XSync (GDK_DISPLAY (), FALSE);
-        gdk_error_trap_pop ();
+                && XkbUseExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &major, &minor);
 
         return have_xkb;
 }
@@ -127,12 +184,11 @@ get_xkb_desc_rec (GsdA11yKeyboardManager *manager)
         Status      status = Success;
 
         gdk_error_trap_push ();
-        desc = XkbGetMap (GDK_DISPLAY (), XkbAllMapComponentsMask, XkbUseCoreKbd);
+        desc = XkbGetMap (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), XkbAllMapComponentsMask, XkbUseCoreKbd);
         if (desc != NULL) {
                 desc->ctrls = NULL;
-                status = XkbGetControls (GDK_DISPLAY (), XkbAllControlsMask, desc);
+                status = XkbGetControls (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), XkbAllControlsMask, desc);
         }
-        XSync (GDK_DISPLAY (), FALSE);
         gdk_error_trap_pop ();
 
         g_return_val_if_fail (desc != NULL, NULL);
@@ -317,8 +373,7 @@ set_server_from_gconf (GsdA11yKeyboardManager *manager,
         }
 
         /* toggle keys */
-        desc->ctrls->ax_options = set_clear (enable_accessX &&
-                                             gconf_client_get_bool (client, CONFIG_ROOT "/togglekeys_enable", NULL),
+        desc->ctrls->ax_options = set_clear (gconf_client_get_bool (client, CONFIG_ROOT "/togglekeys_enable", NULL),
                                              desc->ctrls->ax_options,
                                              XkbAccessXFeedbackMask | XkbAX_IndicatorFBMask);
 
@@ -328,7 +383,7 @@ set_server_from_gconf (GsdA11yKeyboardManager *manager,
         */
 
         gdk_error_trap_push ();
-        XkbSetControls (GDK_DISPLAY (),
+        XkbSetControls (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                         XkbSlowKeysMask         |
                         XkbBounceKeysMask       |
                         XkbStickyKeysMask       |
@@ -342,7 +397,7 @@ set_server_from_gconf (GsdA11yKeyboardManager *manager,
 
         XkbFreeKeyboard (desc, XkbAllComponentsMask, True);
 
-        XSync (GDK_DISPLAY (), FALSE);
+        XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
         gdk_error_trap_pop ();
 
         gnome_settings_profile_end (NULL);
@@ -350,17 +405,19 @@ set_server_from_gconf (GsdA11yKeyboardManager *manager,
 
 static gboolean
 ax_response_callback (GsdA11yKeyboardManager *manager,
+                      GtkWindow              *parent,
                       gint                    response_id,
                       guint                   revert_controls_mask,
                       gboolean                enabled)
 {
+        GConfClient *client;
+        GdkScreen *screen;
         GError *err;
 
         switch (response_id) {
         case GTK_RESPONSE_DELETE_EVENT:
         case GTK_RESPONSE_REJECT:
-        case GTK_RESPONSE_CANCEL: {
-                GConfClient *client;
+        case GTK_RESPONSE_CANCEL:
 
                 client = gconf_client_get_default ();
 
@@ -384,16 +441,19 @@ ax_response_callback (GsdA11yKeyboardManager *manager,
                 g_object_unref (client);
 
                 break;
-        }
+
         case GTK_RESPONSE_HELP:
+                if (!parent)
+                        screen = gdk_screen_get_default ();
+                else
+                        screen = gtk_widget_get_screen (GTK_WIDGET (parent));
+
                 err = NULL;
-                gnome_help_display_desktop (NULL,
-                                            "user-guide",
-                                            "user-guide.xml",
-                                            "goscustaccess-6",
-                                            &err);
-                if (err != NULL) {
-                        GtkWidget *error_dialog = gtk_message_dialog_new (NULL,
+                if (!gtk_show_uri (screen,
+                                   "ghelp:user-guide#goscustaccess-6",
+                                   gtk_get_current_event_time(),
+                                   &err)) {
+                        GtkWidget *error_dialog = gtk_message_dialog_new (parent,
                                                                           0,
                                                                           GTK_MESSAGE_ERROR,
                                                                           GTK_BUTTONS_CLOSE,
@@ -417,7 +477,9 @@ ax_stickykeys_response (GtkDialog              *dialog,
                         gint                    response_id,
                         GsdA11yKeyboardManager *manager)
 {
-        if (ax_response_callback (manager, response_id, XkbStickyKeysMask, manager->priv->stickykeys_shortcut_val)) {
+        if (ax_response_callback (manager, GTK_WINDOW (dialog),
+                                  response_id, XkbStickyKeysMask,
+                                  manager->priv->stickykeys_shortcut_val)) {
                 gtk_widget_destroy (GTK_WIDGET (dialog));
         }
 }
@@ -427,7 +489,9 @@ ax_slowkeys_response (GtkDialog              *dialog,
                       gint                    response_id,
                       GsdA11yKeyboardManager *manager)
 {
-        if (ax_response_callback (manager, response_id, XkbSlowKeysMask, manager->priv->slowkeys_shortcut_val)) {
+        if (ax_response_callback (manager, GTK_WINDOW (dialog),
+                                  response_id, XkbSlowKeysMask,
+                                  manager->priv->slowkeys_shortcut_val)) {
                 gtk_widget_destroy (GTK_WIDGET (dialog));
         }
 }
@@ -443,16 +507,19 @@ maybe_show_status_icon (GsdA11yKeyboardManager *manager)
         show = gconf_client_get_bool (client, CONFIG_ROOT "/enable", NULL);
         g_object_unref (client);
 
+        if (!show && manager->priv->status_icon == NULL)
+                return;
+
+        gsd_a11y_keyboard_manager_ensure_status_icon (manager);
         gtk_status_icon_set_visible (manager->priv->status_icon, show);
 }
 
 #ifdef HAVE_LIBNOTIFY
 static void
 on_notification_closed (NotifyNotification     *notification,
-                        int                     reason,
                         GsdA11yKeyboardManager *manager)
 {
-        /* seems to crash if we unref */
+        g_object_unref (manager->priv->notification);
         manager->priv->notification = NULL;
 }
 
@@ -474,12 +541,11 @@ on_slow_keys_action (NotifyNotification     *notification,
                 return;
         }
 
-        res = ax_response_callback (manager, response_id, XkbSlowKeysMask, manager->priv->slowkeys_shortcut_val);
+        res = ax_response_callback (manager, NULL,
+                                    response_id, XkbSlowKeysMask,
+                                    manager->priv->slowkeys_shortcut_val);
         if (res) {
-                g_signal_handlers_disconnect_by_func (manager->priv->notification, on_notification_closed, manager);
                 notify_notification_close (manager->priv->notification, NULL);
-                g_object_unref (manager->priv->notification);
-                manager->priv->notification = NULL;
         }
 }
 
@@ -501,12 +567,11 @@ on_sticky_keys_action (NotifyNotification     *notification,
                 return;
         }
 
-        res = ax_response_callback (manager, response_id, XkbStickyKeysMask, manager->priv->stickykeys_shortcut_val);
+        res = ax_response_callback (manager, NULL,
+                                    response_id, XkbStickyKeysMask,
+                                    manager->priv->stickykeys_shortcut_val);
         if (res) {
-                g_signal_handlers_disconnect_by_func (manager->priv->notification, on_notification_closed, manager);
                 notify_notification_close (manager->priv->notification, NULL);
-                g_object_unref (manager->priv->notification);
-                manager->priv->notification = NULL;
         }
 }
 
@@ -528,7 +593,7 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         message = _("You just held down the Shift key for 8 seconds.  This is the shortcut "
                     "for the Slow Keys feature, which affects the way your keyboard works.");
 
-        if (! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
+        if (manager->priv->status_icon == NULL || ! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
                 return FALSE;
         }
 
@@ -537,16 +602,15 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         }
 
         if (manager->priv->notification != NULL) {
-                g_signal_handlers_disconnect_by_func (manager->priv->notification, on_notification_closed, manager);
                 notify_notification_close (manager->priv->notification, NULL);
-                g_object_unref (manager->priv->notification);
-                manager->priv->notification = NULL;
         }
 
-        manager->priv->notification = notify_notification_new_with_status_icon (title,
-                                                                                message,
-                                                                                "preferences-desktop-accessibility",
-                                                                                manager->priv->status_icon);
+        gsd_a11y_keyboard_manager_ensure_status_icon (manager);
+        manager->priv->notification = notify_notification_new (title,
+                                                               message,
+                                                               "preferences-desktop-accessibility",
+                                                               NULL);
+        notify_notification_attach_to_status_icon (manager->priv->notification, manager->priv->status_icon);
         notify_notification_set_timeout (manager->priv->notification, NOTIFICATION_TIMEOUT * 1000);
 
         notify_notification_add_action (manager->priv->notification,
@@ -572,10 +636,7 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         if (! res) {
                 g_warning ("GsdA11yKeyboardManager: unable to show notification: %s", error->message);
                 g_error_free (error);
-                g_signal_handlers_disconnect_by_func (manager->priv->notification, on_notification_closed, manager);
                 notify_notification_close (manager->priv->notification, NULL);
-                g_object_unref (manager->priv->notification);
-                manager->priv->notification = NULL;
         }
 
         return res;
@@ -607,10 +668,10 @@ ax_slowkeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
                                                                 0,
                                                                 GTK_MESSAGE_WARNING,
                                                                 GTK_BUTTONS_NONE,
-                                                                title);
+                                                                "%s", title);
 
         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (manager->priv->slowkeys_alert),
-                                                  message);
+                                                  "%s", message);
 
         gtk_dialog_add_button (GTK_DIALOG (manager->priv->slowkeys_alert),
                                GTK_STOCK_HELP,
@@ -625,7 +686,7 @@ ax_slowkeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
         gtk_window_set_title (GTK_WINDOW (manager->priv->slowkeys_alert),
                               _("Slow Keys Alert"));
         gtk_window_set_icon_name (GTK_WINDOW (manager->priv->slowkeys_alert),
-                                  "gnome-dev-keyboard");
+                                  "input-keyboard");
         gtk_dialog_set_default_response (GTK_DIALOG (manager->priv->slowkeys_alert),
                                          GTK_RESPONSE_ACCEPT);
 
@@ -671,7 +732,7 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
                 _("You just pressed two keys at once, or pressed the Shift key 5 times in a row.  "
                   "This turns off the Sticky Keys feature, which affects the way your keyboard works.");
 
-        if (! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
+        if (manager->priv->status_icon == NULL || ! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
                 return FALSE;
         }
 
@@ -680,16 +741,15 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         }
 
         if (manager->priv->notification != NULL) {
-                g_signal_handlers_disconnect_by_func (manager->priv->notification, on_notification_closed, manager);
                 notify_notification_close (manager->priv->notification, NULL);
-                g_object_unref (manager->priv->notification);
-                manager->priv->notification = NULL;
         }
 
-        manager->priv->notification = notify_notification_new_with_status_icon (title,
-                                                                                message,
-                                                                                "preferences-desktop-accessibility",
-                                                                                manager->priv->status_icon);
+        gsd_a11y_keyboard_manager_ensure_status_icon (manager);
+        manager->priv->notification = notify_notification_new (title,
+                                                               message,
+                                                               "preferences-desktop-accessibility",
+                                                               NULL);
+        notify_notification_attach_to_status_icon (manager->priv->notification, manager->priv->status_icon);
         notify_notification_set_timeout (manager->priv->notification, NOTIFICATION_TIMEOUT * 1000);
 
         notify_notification_add_action (manager->priv->notification,
@@ -715,10 +775,7 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         if (! res) {
                 g_warning ("GsdA11yKeyboardManager: unable to show notification: %s", error->message);
                 g_error_free (error);
-                g_signal_handlers_disconnect_by_func (manager->priv->notification, on_notification_closed, manager);
                 notify_notification_close (manager->priv->notification, NULL);
-                g_object_unref (manager->priv->notification);
-                manager->priv->notification = NULL;
         }
 
         return res;
@@ -752,10 +809,10 @@ ax_stickykeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
                                                                   0,
                                                                   GTK_MESSAGE_WARNING,
                                                                   GTK_BUTTONS_NONE,
-                                                                  title);
+                                                                  "%s", title);
 
         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (manager->priv->stickykeys_alert),
-                                                  message);
+                                                  "%s", message);
 
         gtk_dialog_add_button (GTK_DIALOG (manager->priv->stickykeys_alert),
                                GTK_STOCK_HELP,
@@ -770,7 +827,7 @@ ax_stickykeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
         gtk_window_set_title (GTK_WINDOW (manager->priv->stickykeys_alert),
                               _("Sticky Keys Alert"));
         gtk_window_set_icon_name (GTK_WINDOW (manager->priv->stickykeys_alert),
-                                  "gnome-dev-keyboard");
+                                  "input-keyboard");
         gtk_dialog_set_default_response (GTK_DIALOG (manager->priv->stickykeys_alert),
                                          GTK_RESPONSE_ACCEPT);
 
@@ -987,27 +1044,21 @@ register_config_callback (GsdA11yKeyboardManager  *manager,
                           GConfClientNotifyFunc    func,
                           guint                   *notify)
 {
-        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_NONE, NULL);
+        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
         *notify = gconf_client_notify_add (client, path, func, manager, NULL, NULL);
 }
 
-gboolean
-gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
-                                 GError                **error)
+static gboolean
+start_a11y_keyboard_idle_cb (GsdA11yKeyboardManager *manager)
 {
         guint        event_mask;
         GConfClient *client;
-        gboolean     ret = FALSE;
 
         g_debug ("Starting a11y_keyboard manager");
         gnome_settings_profile_start (NULL);
 
-        if (!xkb_enabled (manager)) {
-                g_set_error (error, GSD_KBD_A11Y_ERROR,
-                             GSD_KBD_A11Y_ERROR_NOT_AVAILABLE,
-                             "XKB functionality is disabled.");
+        if (!xkb_enabled (manager))
                 goto out;
-        }
 
         client = gconf_client_get_default ();
 
@@ -1017,6 +1068,14 @@ gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
                                   (GConfClientNotifyFunc) keyboard_callback,
                                   &manager->priv->gconf_notify);
 
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
+	set_devicepresence_handler (manager);
+#endif
+
+        /* Save current xkb state so we can restore it on exit
+         */
+        manager->priv->original_xkb_desc = get_xkb_desc_rec (manager);
+
         event_mask = XkbControlsNotifyMask;
 #ifdef DEBUG_ACCESSIBILITY
         event_mask |= XkbAccessXNotifyMask; /* make default when AXN_AXKWarning works */
@@ -1026,26 +1085,60 @@ gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
         set_server_from_gconf (manager, client);
         g_object_unref (client);
 
-        gdk_error_trap_push ();
-        XkbSelectEvents (GDK_DISPLAY (),
+        XkbSelectEvents (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                          XkbUseCoreKbd,
                          event_mask,
                          event_mask);
 
-        XSync (GDK_DISPLAY (), FALSE);
-        gdk_error_trap_pop ();
-
         gdk_window_add_filter (NULL,
                                (GdkFilterFunc) cb_xkb_event_filter,
                                manager);
 
         maybe_show_status_icon (manager);
 
-        ret = TRUE;
-
  out:
         gnome_settings_profile_end (NULL);
-        return ret;
+
+        return FALSE;
+}
+
+
+gboolean
+gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
+                                 GError                **error)
+{
+        gnome_settings_profile_start (NULL);
+
+        g_idle_add ((GSourceFunc) start_a11y_keyboard_idle_cb, manager);
+
+        gnome_settings_profile_end (NULL);
+
+        return TRUE;
+}
+
+static void
+restore_server_xkb_config (GsdA11yKeyboardManager *manager)
+{
+        gdk_error_trap_push ();
+        XkbSetControls (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                        XkbSlowKeysMask         |
+                        XkbBounceKeysMask       |
+                        XkbStickyKeysMask       |
+                        XkbMouseKeysMask        |
+                        XkbMouseKeysAccelMask   |
+                        XkbAccessXKeysMask      |
+                        XkbAccessXTimeoutMask   |
+                        XkbAccessXFeedbackMask  |
+                        XkbControlsEnabledMask,
+                        manager->priv->original_xkb_desc);
+
+        XkbFreeKeyboard (manager->priv->original_xkb_desc,
+                         XkbAllComponentsMask, True);
+
+        XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
+        gdk_error_trap_pop ();
+
+        manager->priv->original_xkb_desc = NULL;
 }
 
 void
@@ -1055,7 +1148,12 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
 
         g_debug ("Stopping a11y_keyboard manager");
 
-        gtk_status_icon_set_visible (manager->priv->status_icon, FALSE);
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
+        gdk_window_remove_filter (NULL, devicepresence_filter, manager);
+#endif
+
+        if (p->status_icon)
+                gtk_status_icon_set_visible (p->status_icon, FALSE);
 
         if (p->gconf_notify != 0) {
                 GConfClient *client = gconf_client_get_default ();
@@ -1069,6 +1167,10 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
                                   (GdkFilterFunc) cb_xkb_event_filter,
                                   manager);
 
+        /* Disable all the AccessX bits
+         */
+        restore_server_xkb_config (manager);
+
         if (p->slowkeys_alert != NULL)
                 gtk_widget_destroy (p->slowkeys_alert);
 
@@ -1188,17 +1290,28 @@ on_status_icon_activate (GtkStatusIcon          *status_icon,
         }
 }
 
+static void
+gsd_a11y_keyboard_manager_ensure_status_icon (GsdA11yKeyboardManager *manager)
+{
+        gnome_settings_profile_start (NULL);
+
+        if (!manager->priv->status_icon) {
+
+                manager->priv->status_icon = gtk_status_icon_new_from_icon_name ("preferences-desktop-accessibility");
+                g_signal_connect (manager->priv->status_icon,
+                                  "activate",
+                                  G_CALLBACK (on_status_icon_activate),
+                                  manager);
+        }
+
+        gnome_settings_profile_end (NULL);
+}
+
 static void
 gsd_a11y_keyboard_manager_init (GsdA11yKeyboardManager *manager)
 {
         manager->priv = GSD_A11Y_KEYBOARD_MANAGER_GET_PRIVATE (manager);
 
-        manager->priv->status_icon = gtk_status_icon_new_from_icon_name ("preferences-desktop-accessibility");
-        g_signal_connect (manager->priv->status_icon,
-                          "activate",
-                          G_CALLBACK (on_status_icon_activate),
-                          manager);
-
 #ifdef HAVE_LIBNOTIFY
         notify_init ("gnome-settings-daemon");
 #endif /* HAVE_LIBNOTIFY */
