@@ -84,14 +84,14 @@ nautilus_is_running (void)
        gboolean       running;
        gint           error;
 
-       window_id_atom = XInternAtom (GDK_DISPLAY (),
+       window_id_atom = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                      "NAUTILUS_DESKTOP_WINDOW_ID", True);
 
        if (window_id_atom == None) {
                return FALSE;
        }
 
-       retval = XGetWindowProperty (GDK_DISPLAY (),
+       retval = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                     GDK_ROOT_WINDOW (),
                                     window_id_atom,
                                     0,
@@ -118,11 +118,11 @@ nautilus_is_running (void)
                return FALSE;
        }
 
-       wmclass_atom = XInternAtom (GDK_DISPLAY (), "WM_CLASS", False);
+       wmclass_atom = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "WM_CLASS", False);
 
        gdk_error_trap_push ();
 
-       retval = XGetWindowProperty (GDK_DISPLAY (),
+       retval = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                     nautilus_xid,
                                     wmclass_atom,
                                     0,
@@ -180,29 +180,29 @@ draw_background (GsdBackgroundManager *manager,
         for (i = 0; i < n_screens; ++i) {
                 GdkScreen *screen;
                 GdkWindow *root_window;
-                GdkPixmap *pixmap;
+                cairo_surface_t *surface;
 
                 screen = gdk_display_get_screen (display, i);
 
                 root_window = gdk_screen_get_root_window (screen);
 
-                pixmap = gnome_bg_create_pixmap (manager->priv->bg,
-                                                 root_window,
-                                                 gdk_screen_get_width (screen),
-                                                 gdk_screen_get_height (screen),
-                                                 TRUE);
+                surface = gnome_bg_create_surface (manager->priv->bg,
+                                                   root_window,
+                                                   gdk_screen_get_width (screen),
+                                                   gdk_screen_get_height (screen),
+                                                   TRUE);
 
                 if (use_crossfade) {
                         GnomeBGCrossfade *fade;
 
-                        fade = gnome_bg_set_pixmap_as_root_with_crossfade (screen, pixmap);
+                        fade = gnome_bg_set_surface_as_root_with_crossfade (screen, surface);
                         g_signal_connect (fade, "finished",
                                           G_CALLBACK (g_object_unref), NULL);
                 } else {
-                        gnome_bg_set_pixmap_as_root (screen, pixmap);
+                        gnome_bg_set_surface_as_root (screen, surface);
                 }
 
-                g_object_unref (pixmap);
+                cairo_surface_destroy (surface);
         }
 
         gnome_settings_profile_end (NULL);
