@@ -44,6 +44,7 @@
 #include "xutils.h"
 #include "list.h"
 
+#include "gnome-settings-profile.h"
 #include "gsd-clipboard-manager.h"
 
 #define GSD_CLIPBOARD_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_CLIPBOARD_MANAGER, GsdClipboardManagerPrivate))
@@ -81,10 +82,6 @@ typedef struct
         int         offset;
 } IncrConversion;
 
-enum {
-        PROP_0,
-};
-
 static void     gsd_clipboard_manager_class_init  (GsdClipboardManagerClass *klass);
 static void     gsd_clipboard_manager_init        (GsdClipboardManager      *clipboard_manager);
 static void     gsd_clipboard_manager_finalize    (GObject                  *object);
@@ -99,7 +96,6 @@ G_DEFINE_TYPE (GsdClipboardManager, gsd_clipboard_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
-
 /* We need to use reference counting for the target data, since we may
  * need to keep the data around after loosing the CLIPBOARD ownership
  * to complete incremental transfers.
@@ -603,8 +599,11 @@ convert_clipboard (GsdClipboardManager *manager,
                                     &type, &format, &nitems, &remaining,
                                     (unsigned char **) &multiple);
 
-                if (type != XA_ATOM_PAIR)
+                if (type != XA_ATOM_PAIR || nitems == 0) {
+                        if (multiple)
+                                free (multiple);
                         return;
+                }
 
                 for (i = 0; i < nitems; i += 2) {
                         rdata = (IncrConversion *) malloc (sizeof (IncrConversion));
@@ -849,18 +848,19 @@ clipboard_manager_watch_cb (GsdClipboardManager *manager,
         }
 }
 
-gboolean
-gsd_clipboard_manager_start (GsdClipboardManager *manager,
-                             GError             **error)
+static gboolean
+start_clipboard_idle_cb (GsdClipboardManager *manager)
 {
         XClientMessageEvent xev;
 
-        g_debug ("Starting clipboard manager");
+
+        gnome_settings_profile_start (NULL);
 
         init_atoms (manager->priv->display);
 
         /* check if there is a clipboard manager running */
         if (XGetSelectionOwner (manager->priv->display, XA_CLIPBOARD_MANAGER)) {
+                g_warning ("Clipboard manager is already running.");
                 return FALSE;
         }
 
@@ -916,10 +916,23 @@ gsd_clipboard_manager_start (GsdClipboardManager *manager,
                                             0,
                                             NULL);
                 /* FIXME: manager->priv->terminate (manager->priv->cb_data); */
-
-                return FALSE;
         }
 
+        gnome_settings_profile_end (NULL);
+
+        return FALSE;
+}
+
+gboolean
+gsd_clipboard_manager_start (GsdClipboardManager *manager,
+                             GError             **error)
+{
+        gnome_settings_profile_start (NULL);
+
+        g_idle_add ((GSourceFunc) start_clipboard_idle_cb, manager);
+
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
