@@ -61,7 +61,7 @@ gsd_ldsm_dialog_get_checkbutton_text (GsdLdsmDialog *dialog)
         g_return_val_if_fail (GSD_IS_LDSM_DIALOG (dialog), NULL);
         
         if (dialog->priv->other_partitions)
-                return _("Don't show any warnings again for this filesystem");
+                return _("Don't show any warnings again for this file system");
         else
                 return _("Don't show any warnings again");
 }
@@ -210,9 +210,6 @@ gsd_ldsm_dialog_init (GsdLdsmDialog *dialog)
         gtk_window_set_focus_on_map (GTK_WINDOW (dialog), FALSE);
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 
-        /* We don't want a separator - they're really ugly */
-        gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-
         /* Create the image */
         image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
         gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
@@ -254,8 +251,8 @@ gsd_ldsm_dialog_init (GsdLdsmDialog *dialog)
         gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
 						
         /* Set up the action area */
-        gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area), 6);
-        gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 5);
+        gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG (dialog))), 6);
+        gtk_container_set_border_width (GTK_CONTAINER (gtk_dialog_get_action_area (GTK_DIALOG (dialog))), 5);
 	
         gtk_widget_show_all (hbox);
 }
@@ -444,7 +441,7 @@ gsd_ldsm_dialog_new (gboolean     other_usable_partitions,
 	
         if (display_baobab) {
                 button_analyze = gtk_dialog_add_button (GTK_DIALOG (dialog),
-                                                        _("Examine..."),
+                                                        _("Examine…"),
                                                         GSD_LDSM_DIALOG_RESPONSE_ANALYZE);
                 analyze_image = gtk_image_new_from_icon_name ("baobab", GTK_ICON_SIZE_BUTTON);
                 gtk_button_set_image (GTK_BUTTON (button_analyze), analyze_image);
